==> logic-db-0.0.2.tgz: package/docs/sicp/a-boston-high-tech-company.js <==
let logic = require ("../..")

let job = new logic.db_t
let salary = new logic.db_t
let address = new logic.db_t
let supervisor = new logic.db_t


job.i ({
    name: "Bitdiddle Ben",
    dept: "computer",
    role: "wizard",
})

salary.i ({
    name: "Bitdiddle Ben",
    amount: 40000,
})

address.i ({
    name: "Bitdiddle Ben",
    town: "Slunerville",
    road: "Ridge Road",
    door: 10,
})

supervisor.i ({
    slave: "Bitdiddle Ben",
    master: "Warbucks Oliver",
})


address.i ({
    name: "Hacker Alyssa P",
    town: "Cambridge",
    road: "Mass Ave",
    door: 78,
})

job.i ({
    name: "Hacker Alyssa P",
    dept: "computer",
    role: "programmer",
})

salary.i ({
    name: "Hacker Alyssa P",
    amount: 35000,
})

supervisor.i ({
    slave: "Hacker Alyssa P",
    master: "Bitdiddle Ben",
})


address.i ({
    name: "Tweakit Lem E",
    town: "Boston",
    road: "Bay State Road",
    door: 22,
})

job.i ({
    name: "Tweakit Lem E",
    dept: "computer",
    role: "technician",
})

salary.i ({
    name: "Tweakit Lem E",
    amount: 15000,
})

supervisor.i ({
    slave: "Tweakit Lem E",
    master: "Bitdiddle Ben",
})


address.i ({
    name: "Reasoner Louis",
    town: "Slunerville",
    road: "Pine Tree Road",
    door: 80,
})

job.i ({
    name: "Reasoner Louis",
    dept: "computer",
    role: "programmer trainee",
})

salary.i ({
    name: "Reasoner Louis",
    amount: 20000,
})

supervisor.i ({
    slave: "Reasoner Louis",
    master: "Hacker Alyssa P",
})


address.i ({
    name: "Warbucks Oliver",
    town: "Swellesley",
    road: "The Manor",
})

job.i ({
    name: "Warbucks Oliver",
    dept: "administration",
    role: "big wheel",
})

salary.i ({
    name: "Warbucks Oliver",
    amount: 100000,
})

// query

address.query_log (1) ({
    name: "Bitdiddle Ben",
    road: "?road",
    door: "?door",
})

job.query_log (10) ({
    name: "?name",
    dept: "computer",
    role: "?role",
})

let computer_dept_slave = new logic.db_t

computer_dept_slave.i ({
    slave: "?slave",
}) .cond ((the) => {
    let z = new logic.var_t
    return job.o ({ name: the.slave, dept: "computer" })
        .and (supervisor.o ({ slave: the.slave, master: z }))
})

computer_dept_slave.query_log (10) ({
    slave: "?slave",
})

let bigshot = new logic.db_t

bigshot.i ({
    name: "?name",
    dept: "?dept",
}) .cond ((the) => {
    let z = new logic.var_t
    return job.o ({ name: the.name, dept: the.dept })
        .not (supervisor.o ({ slave: the.name, master: z })
              .and (job.o ({ name: z, dept: the.dept })))
})

bigshot.query_log (100) ({
    name: "?name",
    dept: "?dept",
})

let not_so_poor = new logic.db_t

not_so_poor.i ({
    name: "?name",
    amount: "?amount",
}) .cond ((the) => {
    return salary.o ({ name: the.name, amount: the.amount })
        .pred_with_bind ({
            amount: the.amount
        }, (bind) => bind.amount > 30000)
})

not_so_poor.query_log (10) ({
    name: "?name",
    amount: "?amount",
})

==> logic-db-0.0.3.tgz: package/docs/sicp/a-boston-high-tech-company.js <==
let logic = require ("logic-db")

let job = new logic.db_t
let salary = new logic.db_t
let address = new logic.db_t
let supervisor = new logic.db_t

job.i ({
    name: "Bitdiddle Ben",
    dept: "computer",
    role: "wizard",
})

salary.i ({
    name: "Bitdiddle Ben",
    amount: 60000,
})

address.i ({
    name: "Bitdiddle Ben",
    town: "Slumerville",
    road: "Ridge Road",
    door: 10,
})

address.i ({
    name: "Hacker Alyssa P",
    town: "Cambridge",
    road: "Mass Ave",
    door: 78,
})

job.i ({
    name: "Hacker Alyssa P",
    dept: "computer",
    role: "programmer",
})

salary.i ({
    name: "Hacker Alyssa P",
    amount: 40000,
})

supervisor.i ({
    slave: "Hacker Alyssa P",
    master: "Bitdiddle Ben",
})


address.i ({
    name: "Fect Cy D",
    town: "Cambridge",
    road: "Ames Street",
    door: 3,
})

job.i ({
    name: "Fect Cy D",
    dept: "computer",
    role: "programmer",
})

salary.i ({
    name: "Fect Cy D",
    amount: 35000,
})

supervisor.i ({
    slave: "Fect Cy D",
    master: "Bitdiddle Ben",
})


address.i ({
    name: "Tweakit Lem E",
    town: "Boston",
    road: "Bay State Road",
    door: 22,
})

job.i ({
    name: "Tweakit Lem E",
    dept: "computer",
    role: "technician",
})

salary.i ({
    name: "Tweakit Lem E",
    amount: 25000,
})

supervisor.i ({
    slave: "Tweakit Lem E",
    master: "Bitdiddle Ben",
})

address.i ({
    name: "Reasoner Louis",
    town: "Slumerville",
    road: "Pine Tree Road",
    door: 80,
})

job.i ({
    name: "Reasoner Louis",
    dept: "computer",
    role: "programmer trainee",
})

salary.i ({
    name: "Reasoner Louis",
    amount: 30000,
})

supervisor.i ({
    slave: "Reasoner Louis",
    master: "Hacker Alyssa P",
})

supervisor.i ({
    slave: "Bitdiddle Ben",
    master: "Warbucks Oliver",
})

address.i ({
    name: "Warbucks Oliver",
    town: "Swellesley",
    road: "The Manor",
})

job.i ({
    name: "Warbucks Oliver",
    dept: "administration",
    role: "big wheel",
})

salary.i ({
    name: "Warbucks Oliver",
    amount: 150000,
})

address.i ({
    name: "Scrooge Eben",
    town: "Weston",
    road: "Shady Lane",
    door: 10,
})

job.i ({
    name: "Scrooge Eben",
    dept: "accounting",
    role: "chief accountant",
})

salary.i ({
    name: "Scrooge Eben",
    amount: 75000,
})

supervisor.i ({
    slave: "Scrooge Eben",
    master: "Warbucks Oliver",
})


address.i ({
    name: "Cratchet Robert",
    town: "Allston",
    road: "N Harvard Street",
    door: 16,
})

job.i ({
    name: "Cratchet Robert",
    dept: "accounting",
    role: "scrivener",
})

salary.i ({
    name: "Cratchet Robert",
    amount: 18000,
})

supervisor.i ({
    slave: "Cratchet Robert",
    master: "Scrooge Eben",
})

address.i ({
    name: "Aull DeWitt",
    town: "Slumerville",
    road: "Onion Square",
    door: 5,
})

job.i ({
    name: "Aull DeWitt",
    dept: "administration",
    role: "secretary",
})

salary.i ({
    name: "Aull DeWitt",
    amount: 25000,
})

supervisor.i ({
    slave: "Aull DeWitt",
    master: "Warbucks Oliver",
})

let can_do_job = new logic.db_t

can_do_job.i ({
    can: {
        dept: "computer",
        role: "wizard",
    },
    job: {
        dept: "computer",
        role: "programmer",
    },
})

can_do_job.i ({
    can: {
        dept: "computer",
        role: "wizard",
    },
    job: {
        dept: "computer",
        role: "technician",
    },
})

can_do_job.i ({
    can: {
        dept: "computer",
        role: "programmer",
    },
    job: {
        dept: "computer",
        role: "programmer trainee",
    },
})

can_do_job.i ({
    can: {
        dept: "administration",
        role: "secretary",
    },
    job: {
        dept: "administration",
        role: "big wheel",
    },
})

job.query_log (10) ({
    name: "?coder",
    dept: "computer",
    role: "programmer",
})

{
    solutions: [
        { coder: "Hacker Alyssa P" },
        { coder: "Fect Cy D" },
    ]
}

address.query_log (100) ({
    name: "?name",
    town: "?town",
    road: "?road",
    door: "?door",
})

address.query_log (1) ({
    name: "Bitdiddle Ben",
    road: "?road",
    door: "?door",
})

job.query_log (10) ({
    name: "?name",
    dept: "computer",
    role: "?role",
})

let computer_dept_slave = new logic.db_t

computer_dept_slave.i ({
    slave: "?slave",
}) .cond ((the) => {
    let z = new logic.var_t
    return job.o ({ name: the.slave, dept: "computer" })
        .and (supervisor.o ({ slave: the.slave, master: z }))
})

computer_dept_slave.query_log (10) ({
    slave: "?slave",
})

let bigshot = new logic.db_t

bigshot.i ({
    name: "?name",
    dept: "?dept",
}) .cond ((the) => {
    let z = new logic.var_t
    return job.o ({ name: the.name, dept: the.dept })
        .not (supervisor.o ({ slave: the.name, master: z })
              .and (job.o ({ name: z, dept: the.dept })))
})

bigshot.query_log (100) ({
    name: "?name",
    dept: "?dept",
})

let not_so_poor = new logic.db_t

not_so_poor.i ({
    name: "?name",
    amount: "?amount",
}) .cond ((the) => {
    return salary.o ({ name: the.name, amount: the.amount })
        .pred_with_bind ({
            amount: the.amount
        }, (bind) => bind.amount >= 40000)
})

not_so_poor.query_log (10) ({
    name: "?name",
    amount: "?amount",
})
